﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Timers;

namespace PComm
{
	public class PServer
	{
		public static readonly string HB_CHECK = "OK";

		public delegate void ClientDataReceivedHandler(PClientInfo sender, PDataType dataType, byte[] data);
		public event ClientDataReceivedHandler OnDataReceived;

		private PListener listener;
		private Dictionary<string, PClientInfo> clientInfoDict;

		private Timer garbageTimer;

		private object garbageLock = new object();

		public PServer(int port)
		{
			listener = new PListener(port);
			clientInfoDict = new Dictionary<string, PClientInfo>();

			listener.SocketAccepted += Listener_SocketAccepted;

			garbageTimer = new Timer();
			garbageTimer.Interval = 10 * 1000;
			garbageTimer.Elapsed += GarbageTimer_Elapsed;
		}

		public void Start()
		{
			listener.Start();
			garbageTimer.Start();
		}

		private void Listener_SocketAccepted(object sender, SocketAcceptedEventArgs e)
		{
			PClient client = new PClient(e.Accepted);
			client.OnReceived += Client_OnReceived;
			client.OnDisconnected += Client_Disconnected;

			Debug.WriteLine($"[{ DateTime.Now}] Temp client accepted: {client.ID}");
		}

		private void Client_OnReceived(PClient sender, PDataType dataType, byte[] data)
		{
			lock (garbageLock)
			{
				if (!clientInfoDict.ContainsKey(sender.ID))
					clientInfoDict.Add(sender.ID, new PClientInfo());
			}

			DateTime now = DateTime.Now;

			clientInfoDict[sender.ID].Client = sender;
			clientInfoDict[sender.ID].CreatedAt = now;
			clientInfoDict[sender.ID].LastReceivedAt = now;

			clientInfoDict[sender.ID].LastDataType = dataType;
			clientInfoDict[sender.ID].LastData = data;

			// check client id
			if (dataType == PDataType.ClientID)
			{
				string clientId = Encoding.UTF8.GetString(data, 0, data.Length);
				clientInfoDict.Add(clientId, clientInfoDict[sender.ID]);

				lock (garbageLock)
				{
					if (clientInfoDict.ContainsKey(sender.ID))
						clientInfoDict.Remove(sender.ID);
				}

				sender.ID = clientId;
				Debug.WriteLine($"[{ DateTime.Now}] client accepted: {sender.ID}");

				return;
			}

			if (OnDataReceived != null && clientInfoDict.ContainsKey(sender.ID))
				OnDataReceived(clientInfoDict[sender.ID], dataType, data);

			Debug.WriteLine(($"[{ DateTime.Now}] Client message({sender.ID}): {dataType.ToString()}({data.Length})"));
		}

		private void Client_Disconnected(PClient sender)
		{
			if (clientInfoDict.ContainsKey(sender.ID))
			{
				lock (garbageLock)
				{
					clientInfoDict.Remove(sender.ID);
				}
			}
		}

		private void GarbageTimer_Elapsed(object sender, ElapsedEventArgs e)
		{
			List<string> garbageKeys = new List<string>();
			foreach (KeyValuePair<string, PClientInfo> item in clientInfoDict)
			{
				if (!item.Value.Client.Connected)
					garbageKeys.Add(item.Key);
			}

			foreach (string key in garbageKeys)
			{
				clientInfoDict.Remove(key);
			}
		}

		public List<PClientInfo> ClientInfoList
		{
			get { return clientInfoDict.Values.ToList(); }
		}
	}
}
