﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading.Tasks;

namespace Server_Listener.Communication
{
	internal class PClient
	{
		private readonly int BUFF_SIZE = 8192;

		public delegate void ClientReceivedHandler(PClient sender, byte[] data);
		public event ClientReceivedHandler Received;

		public delegate void ClientDisconnectedHandler(PClient sender);
		public event ClientDisconnectedHandler Disconnected;

		public string ID { get; private set; }

		public IPEndPoint EndPoint { get; private set; }

		public bool Connected { get { return socket.Connected; } }

		private Socket socket;

		public PClient(Socket accepted)
		{
			socket = accepted;
			this.ID = Guid.NewGuid().ToString();
			this.EndPoint = (IPEndPoint)socket.RemoteEndPoint;
			socket.BeginReceive(new byte[] { 0 }, 0, 0, 0, AcceptCallback, null);
		}

		private void AcceptCallback(IAsyncResult ar)
		{
			try
			{
				socket.EndReceive(ar);

				byte[] buff = new byte[BUFF_SIZE];
				
				int receiveSize = socket.Receive(buff, buff.Length, SocketFlags.None);
				if (receiveSize <= 0)
				{
					Close();

					if (Disconnected != null)
						Disconnected(this);
				}
				else if (receiveSize < buff.Length)
				{
					Array.Resize<byte>(ref buff, receiveSize);
				}

				if (Received != null)
					Received(this, buff);

				socket.BeginReceive(new byte[] { 0 }, 0, 0, 0, AcceptCallback, null);
			}
			catch (Exception ex)
			{
				this.Close();

				if (Disconnected != null)
					Disconnected(this);

				Debug.WriteLine($"[ERROR] CleintAcceptCallback: {ex.Message}");
			}
		}

		public void Close()
		{
			socket.Close();
			socket.Dispose();
		}
	}
}
